package arm

import "git.urbach.dev/cli/q/src/cpu"

// memory encodes an instruction with a register, a base register and an offset.
func memory(destination cpu.Register, base cpu.Register, imm9 int) uint32 {
	return uint32(imm9&0b1_1111_1111)<<12 | uint32(base)<<5 | uint32(destination)
}

// pair encodes an instruction using a register pair with memory.
func pair(reg1 cpu.Register, reg2 cpu.Register, base cpu.Register, imm7 int) uint32 {
	return uint32(imm7&0b111_1111)<<15 | uint32(reg2)<<10 | uint32(base)<<5 | uint32(reg1)
}

// reg2imm encodes an instruction with 2 registers and an immediate.
func reg2imm(d cpu.Register, n cpu.Register, imm12 int) uint32 {
	return uint32(imm12&0b1111_1111_1111)<<10 | uint32(n)<<5 | uint32(d)
}

// reg3 encodes an instruction with 3 registers.
func reg3(d cpu.Register, n cpu.Register, m cpu.Register) uint32 {
	return uint32(m)<<16 | uint32(n)<<5 | uint32(d)
}

// reg4 encodes an instruction with 4 registers.
func reg4(d cpu.Register, n cpu.Register, m cpu.Register, a cpu.Register) uint32 {
	return uint32(m)<<16 | uint32(a)<<10 | uint32(n)<<5 | uint32(d)
}
